@echo off
cls
REM ***********************************************************
REM Description	: MAME/MESS updating/compiling batch-file
REM Author		: Mucci
REM Date		: 2020-05-23
REM
REM Used external programs (is already in the zip file)
REM				 - Notepad2.exe from http://www.flos-freeware.ch/
REM
REM Changes		: 1.1  updated make comands
REM             : 1.2  merged compiling MAME/MESS
REM             : 1.3  fixed official names MAME/MAMEARCADE/MESS
REM                    creating folders if not exists 
REM             : 1.4  Creating folder if not exists only for choosen version
REM             : 1.5  pause and clean mode added
REM             : 1.6  check if source is installed and with "no" download it first to 'mame'
REM             : 1.7  added "Show last 200 commits in Notepad2"
REM             : 1.8  changed the compile part together, do a tool-update when MAME source is updated
REM             : 2.0  fixed for new compiling tools
REM             : 2.1  some minor changes
REM             : 2.2  updated compiling tools -> that need to update my script
REM             : 2.3  Added the option copy ""additional folders"" to the folder where the compiled exe is
REM             : 2.3a update procedure of the compiling tools improved, (thanks Stiletto)
REM             : 2.4  removed "REGENIE=1 STRIP_SYMBOLS=1" from compile command (less problems)
REM             :      Changed destination folder name  
REM             : 2.5  MAME version finde out uses now makefile
REM             : 2.6  update-tool option implemented
REM             : 2.61 Read the Prozessor value from environment and set it to "-jXX"
REM             : 2.62 STRIP_SYMBOLS=1 added to the compile command
REM ***********************************************************

REM This script is used only for the MAMEdev tools
REM Place this script and Notepad2.exe to your \src\ folder
REM Start always win32env.bat first (Environment) and after that "compile-MAME.cmd"
REM ------------------------------------------------------------------------------------------------------
REM Check on which OS is runing this script (x32/x64) and set the variable
if defined PROGRAMFILES(X86) (
    set CompileX=x64
) else (
    set CompileX=x86
)
REM set the value for Architecture: x86 or x64 (default=x64) -> if you want to override the autotetection
REM set CompileX=x64

REM Set the default values for some settings
REM
REM define version to compile as default = MAME, MAMEARCADE or MESS (default=MAME)
set Compileversion=MAME
REM CPU Core -> change that to the amount of cpu-core you have and add 1 -> "core +1"
REM Example: dual-core or quad-core processor -> use the option -j3 or -j5
set /a THREADS=%NUMBER_OF_PROCESSORS% + 1
set Compilecpu=j%THREADS%
REM set Compilecpu=j13
REM Compile with tools: 0/off/without or 1/on/with (default=0/off/without)
set Compiletools=0
set Compiletoolsstring=off
set Compiletoolsstring2=without
REM set default values for pause: on or off (default=on)
set batchpause=on
REM set default values for doing a clean before compiling: on or off (default=off)
set cleanmode=off
REM set default value for addon folder
set Addonfolder=off
REM set default value for updating tools
set updatetools=on
REM ------------------------------------------------------------------------------------------------------

REM Displaying a start information
ECHO    -----------------------------------------------
ECHO      MAMEdev update/compiling batch-file starting
ECHO    -----------------------------------------------
ECHO         (c) 2020 mameguide.info
ECHO.
ECHO.

REM Check if Mame source is installed or not
IF EXIST %~dp0\mame\ GOTO JM-checkvar
ECHO    ---------------------------------------
ECHO      downloading MAME Source
ECHO    ---------------------------------------
ECHO.
REM Copy the source from MAMEDev using the git command
git clone https://github.com/mamedev/mame.git
cd %~dp0\mame\
REM Set Build variable
set /p Build=<%~dp0\mame\.git\refs\heads\master
REM Write found build numbers to a file and convert it to the revision number
git rev-list %Build% --count >%~dp0\mame\revision.txt
REM Write Version number to a file
FINDSTR /C:"'#define BARE_BUILD_VERSION " %~dp0\mame\makefile >version_raw.txt
set /p MAMEVERSION=<version_raw.txt
set MAMEVERSION=%MAMEVERSION:~36,-7%
echo.%MAMEVERSION%>version.txt
set /p MAMEVERSION=<%~dp0\mame\version.txt
cd %~dp0\

:JM-checkvar
REM Set variable read from the source folder (version.txt and revision.txt)
ECHO    ---------------------------------------
ECHO      set some settings
ECHO    ---------------------------------------
ECHO.
cd %~dp0\mame\
REM set version variable read from a file
set /p MAMEVERSION=<%~dp0\mame\version.txt
REM set revision variable read from a file
set /p Revision=<%~dp0\mame\revision.txt
cd %~dp0\

REM Main menu
:JM-MENU
cls
ECHO    ---------------------------------------
ECHO      MAMEdev update and compiling V2.62
ECHO    ---------------------------------------
ECHO         (c) 2020 mameguide.info
ECHO    ---------------------------------------
ECHO.
ECHO    [U] = Update local MAME source folder
ECHO    ---------------------------------------
ECHO    [C] = Compile MAME:
ECHO          - (V) Version		  : %Compileversion%
ECHO          - (A) Architecture	  : %CompileX%
ECHO          - (T) with Tools	  : %Compiletoolsstring%
ECHO          - (F) add Folderstructure: %Addonfolder%
ECHO    ---------------------------------------
ECHO    (B) = Before compiling do a clean: %cleanmode%
ECHO    (P) = Batch Pause Mode           : %batchpause%
ECHO    (Z) = Update compile tools       : %updatetools%
ECHO    ---------------------------------------
ECHO    [S] = Show last 200 commits in Notepad2
ECHO    ---------------------------------------
ECHO    [E] = Exit
ECHO    ---------------------------------------
ECHO.
ECHO    ---------------------------------------
ECHO    Note: When you having problems with 
ECHO          compiling and tools are not -/+ 
ECHO          generated - enable "clean" !!!!
ECHO    ---------------------------------------
ECHO.
REM jump to the chosen part
CHOICE /C UCVATFBPSEZ /N /M "Command: "
IF ERRORLEVEL 11 GOTO JM-Toolsupdate
IF ERRORLEVEL 10 GOTO JM-EXIT
IF ERRORLEVEL 9 GOTO JM-ShowLog
IF ERRORLEVEL 8 GOTO JM-PauseMode
IF ERRORLEVEL 7 GOTO JM-CleanMode
IF ERRORLEVEL 6 GOTO JM-Addonfolder
IF ERRORLEVEL 5 GOTO JM-Compiletools
IF ERRORLEVEL 4 GOTO JM-Compilearchitecture
IF ERRORLEVEL 3 GOTO JM-Compileversion
IF ERRORLEVEL 2 GOTO JM-Compiling
IF ERRORLEVEL 1 GOTO JM-UPDATESRC

REM Check the Pause Mode and set it to off or on
:JM-PauseMode
if [%batchpause%] EQU [off] goto JM-PauseMode1
if [%batchpause%] EQU [on] goto JM-PauseMode2
:JM-PauseMode1
set batchpause=on
GOTO JM-MENU
:JM-PauseMode2
set batchpause=off
GOTO JM-MENU

REM Check the Clean Mode and set it to off or on
:JM-CleanMode
if [%cleanmode%] EQU [off] goto JM-CleanMode1
if [%cleanmode%] EQU [on] goto JM-CleanMode2
:JM-CleanMode1
set cleanmode=on
GOTO JM-MENU
:JM-CleanMode2
set cleanmode=off
GOTO JM-MENU

REM Set the version to compile
:JM-Compileversion
if [%Compileversion%] EQU [MAME] goto JM-Compileversion1
if [%Compileversion%] EQU [ARCADE] goto JM-Compileversion2
if [%Compileversion%] EQU [MESS] goto JM-Compileversion3
:JM-Compileversion1
set Compileversion=ARCADE
GOTO JM-MENU
:JM-Compileversion2
set Compileversion=MESS
GOTO JM-MENU
:JM-Compileversion3
set Compileversion=MAME
GOTO JM-MENU

REM Set the architecture to compile
:JM-Compilearchitecture
if [%CompileX%] EQU [x86] goto JM-Compilearchitecture1
if [%CompileX%] EQU [x64] goto JM-Compilearchitecture2

:JM-Compilearchitecture1
set CompileX=x64
Rem Check if it is a x86 architecture, in that case x64 is not possible
if defined PROGRAMFILES(X86) (
    set CompileX=x64
) else (
    set CompileX=x86
)
REM Check if the x64 compiling tools are installed
IF NOT EXIST ..\mingw64\ set CompileX=x86
GOTO JM-MENU

:JM-Compilearchitecture2
REM Check if the Compining-tools for x32 is available and set the value, if not available it can not be changed to x86
set CompileX=x86
IF NOT EXIST ..\mingw32\ set CompileX=x64
IF NOT EXIST ..\win32\config32.bat set CompileX=x64
GOTO JM-MENU

REM compile with tools or not
:JM-Compiletools
if [%Compiletools%] EQU [0] goto JM-Compiletools1
if [%Compiletools%] EQU [1] goto JM-Compiletools2
:JM-Compiletools1
set Compiletools=1
set Compiletoolsstring=on
set Compiletoolsstring2=with
ECHO    ------------------------------------------------
ECHO      enabling/disabling tools requires a clean up
ECHO    ------------------------------------------------
ECHO.
cd %~dp0\mame
make.exe clean
cd %~dp0\
GOTO JM-MENU
:JM-Compiletools2
set Compiletools=0
set Compiletoolsstring=off
set Compiletoolsstring2=without
ECHO    ------------------------------------------------
ECHO      enabling/disabling tools requires a clean up
ECHO    ------------------------------------------------
ECHO.
cd %~dp0\mame
make.exe clean
cd %~dp0\
GOTO JM-MENU

:JM-Addonfolder
REM Check the addonfolder value if it i off or on
if [%Addonfolder%] EQU [off] goto JM-Addonfolder1
if [%Addonfolder%] EQU [on] goto JM-Addonfolder2
:JM-Addonfolder1
set Addonfolder=on
GOTO JM-MENU
:JM-Addonfolder2
set Addonfolder=off
GOTO JM-MENU

:JM-Toolsupdate
REM set the update option for the compile tools to off or on
if [%updatetools%] EQU [off] goto JM-Toolsupdate1
if [%updatetools%] EQU [on] goto JM-Toolsupdate2
:JM-Toolsupdate1
set updatetools=on
GOTO JM-MENU
:JM-Toolsupdate2
set updatetools=off
GOTO JM-MENU

:JM-UPDATESRC
cls
if [%updatetools%] EQU [off] goto JM-UPDATESRC2
ECHO    ---------------------------------------
ECHO      Updating local MAME tool folder
ECHO    ---------------------------------------
ECHO.
cd ..
pacman -Sy
pacman -S bash pacman msys2-runtime --noconfirm --needed
pacman -Su --noconfirm
:JM-UPDATESRC2
cd %~dp0\mame
ECHO    ---------------------------------------
ECHO      Updating local MAME source folder
ECHO    ---------------------------------------
ECHO.
REM download the update from the repository
git.exe pull -v --no-rebase --progress "origin"
REM Read the Build number from the file
set /p Build=<%~dp0\mame\.git\refs\heads\master
REM Ask the repository with the build number what revision number this is
git rev-list %Build% --count >%~dp0\mame\revision.txt
set /p Revision=<%~dp0\mame\revision.txt
REM Write Version number to a file
FINDSTR /C:"'#define BARE_BUILD_VERSION " %~dp0\mame\makefile >version_raw.txt
set /p MAMEVERSION=<version_raw.txt
set MAMEVERSION=%MAMEVERSION:~36,-7%
echo.%MAMEVERSION%>version.txt
set /p MAMEVERSION=<%~dp0\mame\version.txt
REM Display information before going back to main menu
ECHO.
ECHO    ---------------------------------------
ECHO      Version	:  %MAMEVERSION%
ECHO      Build	:  %Build%
ECHO      Revision	:  %Revision%
ECHO    ---------------------------------------
ECHO.
cd %~dp0\
REM do a pause if it is set
if [%batchpause%] EQU [on] pause
GOTO JM-MENU

:JM-Compiling
cls
ECHO    ------------------------------------------
ECHO      Compiling now %Compileversion% %CompileX% %Compiletoolsstring2% tools
ECHO    ------------------------------------------
ECHO.
REM Check if x32/x64 version of the compiling tools is installed
IF NOT EXIST ..\win32\config64.bat GOTO JM-Folder1
IF NOT EXIST ..\mingw32\ GOTO JM-Folder1
cd ..\win32\
if [%CompileX%] EQU [x64] @call config64.bat
if [%CompileX%] EQU [x86] @call config32.bat
:JM-Folder1
cd %~dp0\MAME\
REM Check if folder Mame_builds exists or not
IF EXIST %~dp0\_MAME\%Compileversion%\ GOTO JM-Folder2
ECHO    ------------------------------------------
ECHO      creating folder  %~dp0\_MAME\%Compileversion%\
ECHO    ------------------------------------------
ECHO.
mkdir %~dp0\_MAME\%Compileversion%\
:JM-Folder2
REM first do a cleanup and then compiling with that options
if [%cleanmode%] EQU [on] make.exe clean
if [%Compiletools%] EQU [0] goto JM-Folder3
REM compile the right version with tools
if [%Compileversion%] EQU [MAME] make.exe -%Compilecpu% TOOLS=1 ARCHITECTURE=_%CompileX% STRIP_SYMBOLS=1
if [%Compileversion%] EQU [ARCADE] make.exe -%Compilecpu% TOOLS=1 SUBTARGET=arcade ARCHITECTURE=_%CompileX% STRIP_SYMBOLS=1
if [%Compileversion%] EQU [MESS] make.exe -%Compilecpu% TOOLS=1 SUBTARGET=mess ARCHITECTURE=_%CompileX% STRIP_SYMBOLS=1
goto JM-Folder4
:JM-Folder3
REM compile the right version without tools
if [%Compileversion%] EQU [MAME] make.exe -%Compilecpu% ARCHITECTURE=_%CompileX% STRIP_SYMBOLS=1
if [%Compileversion%] EQU [ARCADE] make.exe -%Compilecpu% SUBTARGET=arcade ARCHITECTURE=_%CompileX% STRIP_SYMBOLS=1
if [%Compileversion%] EQU [MESS] make.exe -%Compilecpu% SUBTARGET=mess ARCHITECTURE=_%CompileX% STRIP_SYMBOLS=1
:JM-Folder4
cd %~dp0\
REM set foldername from the version and revision that is set from the source folder
set foldername=%Mameversion%r%Revision%%CompileX%
REM check if folder does exists -> if yes delete it with content
IF EXIST %~dp0\_MAME\%Compileversion%\%foldername% rmdir /Q /S %~dp0\_MAME\%Compileversion%\%foldername%
REM create a foldername with that information 
mkdir %~dp0\_MAME\%Compileversion%\%foldername%
REM Move all executables from the source folder to this new subfolder
ECHO    ------------------------------------------
ECHO      moving created files to %~dp0\_MAME\%Compileversion%\%foldername%\
ECHO    ------------------------------------------
ECHO.
move /Y "%~dp0\Mame\*.exe" "%~dp0\_MAME\%Compileversion%\%foldername%\"
REM copy addon folder/files when enabled
if [%Addonfolder%] EQU [off] goto JM-Folder5
ECHO    ------------------------------------------
ECHO      copying now addon folder structure 
ECHO    ------------------------------------------
ECHO.
xcopy /E "%~dp0\Mame\artwork" "%~dp0\_MAME\%Compileversion%\%foldername%\artwork\" >NUL
xcopy /E "%~dp0\Mame\bgfx" "%~dp0\_MAME\%Compileversion%\%foldername%\bgfx\" >NUL
xcopy /E "%~dp0\Mame\docs" "%~dp0\_MAME\%Compileversion%\%foldername%\docs\" >NUL
xcopy /E "%~dp0\Mame\hash" "%~dp0\_MAME\%Compileversion%\%foldername%\hash\" >NUL
xcopy /E "%~dp0\Mame\hlsl" "%~dp0\_MAME\%Compileversion%\%foldername%\hlsl\" >NUL
xcopy /E "%~dp0\Mame\ini" "%~dp0\_MAME\%Compileversion%\%foldername%\ini\" >NUL
xcopy /E "%~dp0\Mame\language" "%~dp0\_MAME\%Compileversion%\%foldername%\language\" >NUL
xcopy /E "%~dp0\Mame\nl_examples" "%~dp0\_MAME\%Compileversion%\%foldername%\nl_examples\" >NUL
xcopy /E "%~dp0\Mame\plugins" "%~dp0\_MAME\%Compileversion%\%foldername%\plugins\" >NUL
mkdir "%~dp0\_MAME\%Compileversion%\%foldername%\roms" >NUL
xcopy /E "%~dp0\Mame\samples" "%~dp0\_MAME\%Compileversion%\%foldername%\samples\" >NUL
:JM-Folder5
REM do a pause if it is set
if [%batchpause%] EQU [on] pause
GOTO JM-MENU

:JM-ShowLog
cls
cd %~dp0\MAME\
REM Read the Build number from the local file
set /p Build=<%~dp0\MAME\.git\refs\heads\master
REM Read the version number from the local file
set /p Revision=<%~dp0\MAME\revision.txt
REM read the last entry from the local file
set /p MAMEVERSION=<%~dp0\mame\version.txt
REM read the commit number from the remote source (master) and write it to a file
git ls-remote https://github.com/mamedev/mame.git master >%~dp0\MAME\remotebuild.txt
REM Set variable from the written file
set /P Remotebuild=<%~dp0\MAME\remotebuild.txt
REM cut everything above the 40 position nad set it again as variable
set Remotebuild=%Remotebuild:~0,40%
rem git rev-list %Remotebuild% --count >revisionremote.txt
rem set /p Revisionremote=<revisionremote.txt
ECHO    ---------------------------------------
ECHO     Show last 200 commits in Notepad2
ECHO    ---------------------------------------
ECHO	 Local source version:
ECHO.
ECHO     Version	:  %MAMEVERSION%
ECHO     Build	:  %Build%
ECHO     Revision	:  %Revision%
ECHO    ---------------------------------------
ECHO.
REM Check if remote source has the same commit number than local
If %Build%==%Remotebuild% GOTO JM-SHOWLOG1
ECHO     local source is not up-to-date
ECHO     Remote source code is at %Remotebuild%
ECHO    ---------------------------------------
ECHO.
GOTO JM-SHOWLOG2
:JM-SHOWLOG1
ECHO     local source is up-to-date with remote
ECHO    ---------------------------------------
ECHO.
:JM-SHOWLOG2
REM write the last 200 commits to a file
rem git.exe log -200 --pretty=format:"Commit : %H"
git.exe log -200 >%~dp0\MAME\log.txt
rem "%nThe author was %an%nDate: %ar%nThe title: %s%n----------------------------------------------"
REM >log.txt
cd %~dp0\
REM do a pause if it is set
if [%batchpause%] EQU [on] pause
REM Open Notepad2 with the log-file
%~dp0\notepad2.exe %~dp0\MAME\log.txt
REM Remove temporary files (not needed for other tasks)
del %~dp0\MAME\log.txt
del %~dp0\MAME\remotebuild.txt
del %~dp0\MAME\revisionremote.txt
GOTO JM-MENU

:JM-EXIT